#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Les algorithmes, outils de décision automatique.
# 
# L'exemple du trading algorithmique.

# ## Introduction

# ### Plan

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu(last_level=2, menu_id="main")


# ### A propos de moi
# 
# * Senior Data Scientist chez Microsoft
# * Professeur à l'ENSAE depuis 2000
# * Professeur dans la voie data science depuis 2014
# * Formation Data Science de l'Institut des Actuaires
# * [lesenfantscodaient.fr](http://lesenfantscodaient.fr)
# * Passage en finance (hedge funds), travail sur des algorithmes de trading automatiques (2007)

# ### A qui s'adresse cette présentation
# 
# * au plus grand nombre

# ## Raccourcis d'histoire financière et technique
# 
# * On débute en 1971.
# * L'informatique et les mathématiques.
# * Automatisation.

# ### Bretton Woods 1971 : le début de la technique
# 
# * Source : [Comment j'ai détesté les maths](http://www.arte.tv/guide/fr/068432-000-A/comment-j-ai-deteste-les-maths)
# * Arrêt de la politique du taux de change unique.
# * Volonté des sociétés multinationales de s'assurer contre le risque de change.
# * Début de la modélisation mathématique.

# ### Informatisation
# 
# * 1984 : dématérialisation des titres à la bourse française
# * 1986 : CAC (Compagnie de Agents de Changes) devient la *Cotation Assistée en Continu*.
# * 1988 : les ordres sont tous passés depuis des ordinateurs
# * 1997 : les ECN (Electronic Communication  Network) sont conçus. Il s'agit de centres informatiques qui mettent en relation les professionnels et les brokers pour effectuer des transactions informatisées.
# * 1997-2002 : premiers algorithmes de trading
# * 2003 : début du trading haute fréquence
# * 2010 : [Flash Crash](https://fr.wikipedia.org/wiki/Flash_Crash_de_2010), probablement dû au trading haute fréquence

# ### La finance est numérisée
# 
# * Un des premiers domaines économique entièrement numérisée.
# * Un des plus profitables.
# * Capable de recruter les meilleurs talents.

# ### Hedge Fund
# 
# * [Renaissance Technologies](https://fr.wikipedia.org/wiki/Renaissance_Technologies)
# * [CFM](https://fr.wikipedia.org/wiki/Capital_Fund_Management)
# 
# Ces sociétés emploient beaucoup de docteurs en mathématiques et physiques.

# ### Formation
# 
# * [Master Probabilités et Finance](http://finance.math.upmc.fr/index2.php) surnommé le master El Karoui
# 
# Et tout autour du monde et principalement dans les universités anglo-saxonnes :
# 
# * [MS in Financial Engineering](http://ieor.columbia.edu/ms-financial-engineering)

# ### Automatisation possible
# 
# Tout est prêt pour l'automatisation
# 
# * Tout est numérique.
# * Les meilleurs ingénieurs.
# * Les meilleurs finances. 
# * Le retour sur investissement est très rapide.

# ### Intérêt de l'automatisation
# 
# * Traiter plus de données.
# * Traiter autant de données mais plus vite.
# 
# Pour être meilleur que le concurrent. 

# ### Problème à taille humaine
# 
# Combien avez-vous d'amis sur Facebook ? 
# 
# [La règle des 150](https://en.wikipedia.org/wiki/The_Tipping_Point).

# ## Un cas concret
# 
# * Un exemple d'algorithme de trading algorithmique : trend following.
# * Notion de back test
# * Trading haute fréquence ?
# * Machine learning
# * Pas un inventaire des stratégies existantes.

# ### Une série financière

# In[2]:


from pyensae.finance import StockPrices
prices = StockPrices(tick="MSFT", url='yahoo_new')
prices.dataframe.tail()


# In[3]:


get_ipython().run_line_magic('matplotlib', 'inline')


# In[4]:


prices.plot(figsize=(14,4));


# In[5]:


prices[-20:].plot(figsize=(14,4), field="ohlc");


# ### Stratégie : trend following
# 
# Le principe : [trend following](https://en.wikipedia.org/wiki/Trend_following).

# In[6]:


from pyquickhelper.helpgen import NbImage
NbImage("images/trend.png", width=800)


# ### La mise au point
# 
# De nombreux paramètres :
# 
# * La longueur de la [moyenne mobile](https://fr.wikipedia.org/wiki/Moyenne_glissante).
# * La hauteur des [bandes de Bollinger](https://fr.wikipedia.org/wiki/Bandes_de_Bollinger).
# * La somme investie.
# * ...
# 
# Les stratégies ont souvent plus d'une dizaine de paramètres.

# Deux paramètres en rouge : longueur de la moyenne mobile et la hauteur des bandes de Bollinger

# In[7]:


NbImage("images/trendp.png", width=400)


# ### Optimisation
# 
# * Choisir les meilleurs paramètres
# * [Backtest](https://en.wikipedia.org/wiki/Backtesting) : simuler la performance de la stratégie sur le passé

# In[8]:


NbImage("images/backtest.png", width=400)


# Comparaison de plusieurs jeux de paramètres (source [Backtesting a Simple Stock Trading Strategy](https://www.r-bloggers.com/backtesting-a-simple-stock-trading-strategy/))

# In[9]:


NbImage("images/backtest2.png", width=500)


# ### Tests exhaustifs
# 
# * Tester de nombreux jeux de paramètres
# * Tester sur un grand nombre de séries financières
# * Tester la résistance aux crises
# * Prise en compte de défaut de simulation : coût de transaction, slippage

# ### Portefeuille de stratégies
# 
# * Implémenter une seule stratégie est risqué.
# * Construction de portefeuilles de stratégies (plus d'une centaine)

# ### Importance de l'informatique
# 
# * Puissance de calcul
# * Agilité
# * Passage en production
# * Résistance aux pannes
# * Utilisation de [stop-order](http://www.investopedia.com/university/intro-to-order-types/stop-orders.asp)

# ### Trading daily, intraday
# 
# * Daily : calcul des décisions chaque jour
# * Intraday : calcul des décisions chaque minute
# * Haute Fréquence : calcul d'une décision en quelques millisecondes
# 
# Plus la vitesse de trading est importante, plus l'informatique joue un grand rôle, plus le boulot est technique.
# 
# * La course aux millisecondes : [Hibernia Network](https://en.wikipedia.org/wiki/Hibernia_Networks).

# ## Réflexions sur la finance
# 
# * Machine learning, exploitation des news, signaux autres que financiers
# * Côté éthique
# * Parallèle avec le real-time-bidding

# ### Pourquoi le machine learning ?
# 
# * **Back test** : méthode issue du machine learning
# * **Boîte noire** : une façon d'utiliser l'information qu'on ne sait pas utiliser de manière évidente
# * **Traitement des news** : celle-ci ont une influence non négligeable sur les cours mais ce n'est pas une informatique numérique.
# * **Deep learning** : le deep learning fonctionne sur des images, du son, du texte, pourquoi pas sur des cours financiers.

# ### Ethique
# 
# * Trading haute fréquence : quelques millisecondes d'avance sur les concurrents
# * plus de 50% des échanges issus du trading haute fréquence
# * Une plus grande fluidité (possibilité d'acheter et de vendre à tout moment)
# * Quelle utilité économique ?
# * Recrutement des meilleurs talents

# ### Un parallèle avec Bitcoin
# 
# * Le principe de [Bitcoin](https://fr.wikipedia.org/wiki/Bitcoin) : miner
# * La course à la puissance

# ### Un parallèle avec le Real Time Bidding
# 
# * [Real Time Bidding](https://fr.wikipedia.org/wiki/Real_Time_Bidding), plate-formes d'échanges, enchères
# * Publicité sur Internet
# * Un champion national : Criteo
# * Mécanisme des enchères
# * Différence avec la finance : utilisation des données

# ### Ingéniérisation de la finance
# 
# * Popularisation du trading automatique.
# * La finance n'est plus aussi attractive.
# * Des traders qui passent de la finance à la data science.

# ## Conclusion
# 
# * Le rôle prépondérant de l'informatique
# * L'exécution n'est plus manuelle, trop rapide, en trop grand nombre
# * Le savoir est converti sous forme d'algorithmes.
# * La finance en dehors de la finance : [Chine: la finance de l'ombre atteint un record](http://lexpansion.lexpress.fr/actualites/1/actualite-economique/chine-la-finance-de-l-ombre-atteint-un-record_1899027.html)

# In[10]: